@@ -2,22 +2,21 @@
 #
 # read.cross.R
 #
-# copyright (c) 2000-8, Karl W Broman
-# last modified Jun, 2008
+# copyright (c) 2000-9, Karl W Broman
+# last modified Apr, 2009
 # first written Aug, 2000
 #
 #     This program is free software; you can redistribute it and/or
-#     modify it under the terms of the GNU General Public License, as
-#     published by the Free Software Foundation; either version 2 of
-#     the License, or (at your option) any later version. 
+#     modify it under the terms of the GNU General Public License,
+#     version 3, as published by the Free Software Foundation.
 # 
 #     This program is distributed in the hope that it will be useful,
 #     but without any warranty; without even the implied warranty of
-#     merchantability or fitness for a particular purpose.  See the
-#     GNU General Public License for more details.
+#     merchantability or fitness for a particular purpose.  See the GNU
+#     General Public License, version 3, for more details.
 # 
-#     A copy of the GNU General Public License is available at
-#     http://www.r-project.org/Licenses/
+#     A copy of the GNU General Public License, version 3, is available
+#     at http://www.r-project.org/Licenses/GPL-3
 #
 # Part of the R/qtl package
 # Contains: read.cross, fixXgeno.bc, fixXgeno.f2, asnumericwithdec
@@ -131,8 +130,10 @@ function(format=c("csv", "csvr", "csvs", "csvsr", "mm", "qtx",
   if(any(chrtype=="X") && convertXdata) {
     if(class(cross)[1]=="bc")
       cross <- fixXgeno.bc(cross)
-    if(class(cross)[1]=="f2")
-      cross <- fixXgeno.f2(cross)
+    if(class(cross)[1]=="f2") {
+      if(missing(alleles)) alleles <- c("A","B")
+      cross <- fixXgeno.f2(cross, alleles)
+    }
   }
 
   # re-estimate map?
@@ -258,7 +259,7 @@ function(cross)
 # fixXgeno.f2: fix up the X chromosome genotype data for intercross
 ##############################
 fixXgeno.f2 <-
-function(cross)
+function(cross, alleles)
 {
   omitX <- FALSE
 
@@ -270,6 +271,14 @@ function(cross)
   # find "sex" and "pgm" in the phenotype data
   sexpgm <- getsex(cross)
 
+  AA <- paste(rep(alleles[1], 2), collapse="")
+  AB <- paste(alleles, collapse="")
+  BB <- paste(rep(alleles[2], 2), collapse="")
+  cross0 <- paste("(", alleles[1], "x", alleles[2], ")x(",
+                  alleles[1], "x", alleles[2], ")", sep="")
+  cross1 <- paste("(", alleles[2], "x", alleles[1], ")x(",
+                  alleles[2], "x", alleles[1], ")", sep="")
+
   if(!is.null(sexpgm$sex) && !is.null(sexpgm$pgm)) {
     # both "sex" and "pgm" are provided
 
@@ -307,12 +316,14 @@ function(cross)
       }
       if(any(!is.na(femalegeno0) & femalegeno0==3)) {
         n.omit <- sum(!is.na(femalegeno0) & femalegeno0==3)
-        warning(" --Omitting ", n.omit, " BB genotypes from females from cross (AxB)x(AxB) on the X chr.\n")
+        warning(" --Omitting ", n.omit, " ", BB, " genotypes from females from cross ",
+                cross0, " on the X chr.\n")
         femalegeno0[!is.na(femalegeno0) & femalegeno0==3] <- NA
       }
       if(any(!is.na(femalegeno1) & femalegeno1==1)) {
         n.omit <- sum(!is.na(femalegeno1) & femalegeno1==1)
-        warning(" --Omitting ", n.omit, " AA genotypes from females from cross (BxA)x(BxA) on the X chr.\n")
+        warning(" --Omitting ", n.omit, " ", AA, " genotypes from females from cross ",
+                cross1, " on the X chr.\n")
         femalegeno1[!is.na(femalegeno1) & femalegeno1==1] <- NA
       }
       femalegeno1[!is.na(femalegeno1) & femalegeno1==3] <- 1
