@@ -3,7 +3,7 @@
 # scanone.R
 #
 # copyright (c) 2001-9, Karl W Broman
-# last modified Apr, 2009
+# last modified Sep, 2009
 # first written Feb, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -121,10 +121,64 @@ function(cross, chr, pheno.col=1, model=c("normal","binary","2part","np"),
     }
   }
 
-  # multiple phenotype for methods except imp and hk or when there
-  #     are some individuals missing only a portion of the phenotypes
+  # use all observations; not in a permutation test; different phenotypes have different sets of missing values
+  #   -> want to do in batches, but need to define batches by the pattern of missing data
+  if(n.perm <= 0 && use=="all.obs" && length(pheno.col) > 1 && (method=="hk" || method=="imp")) { 
+    # drop individuals with missing covariates
+    cross$pheno <- cbind(cross$pheno, rep(1, nind(cross)))
+    temp <- checkcovar(cross, nphe(cross), addcovar, intcovar,
+                         perm.strata, TRUE)
+    cross <- temp[[1]]
+    pheno <- cross$pheno[,pheno.col, drop=FALSE]
+    addcovar <- temp[[3]]
+    intcovar <- temp[[4]]
+    n.addcovar <- temp[[5]]
+    n.intcovar <- temp[[6]]
+    perm.strata <- temp[[7]]
+
+    # determine the batches (defined by the pattern of missing data)
+    patterns <- apply(pheno, 2, function(a) paste(!is.na(a), collapse=":"))
+    upat <- unique(patterns)
+    m <- match(patterns, upat)
+    batches <- vector("list", length(upat))
+    upat <- lapply(strsplit(upat, ":"), function(a) as.logical(a))
+    for(i in seq(along=batches)) batches[[i]] <- pheno.col[m==i]
+
+    # run scanone for one batch at a time
+    out <- NULL
+    for(i in seq(along=batches)) {
+      if(!is.null(addcovar)) {
+        if(!is.matrix(addcovar)) addcovar <- as.matrix(addcovar)
+      tempac <- addcovar[upat[[i]],,drop=FALSE]
+      }
+      else tempac <- addcovar
+      if(!is.null(intcovar)) {
+        if(!is.matrix(intcovar)) intcovar <- as.matrix(intcovar)
+        tempic <- intcovar[upat[[i]],,drop=FALSE]
+      }
+      else tempic <- intcovar
+
+      temp <- scanone(subset(cross, ind=upat[[i]]), chr=chr, pheno.col=batches[[i]], model=model,
+                      method=method, addcovar=tempac, intcovar=tempic,
+                      weights=weights, use=use, upper=upper, ties.random=ties.random,
+                      start=start, maxit=maxit, tol=tol, n.perm=n.perm, perm.Xsp=perm.Xsp,
+                      perm.strata=perm.strata, verbose=verbose, batchsize=batchsize,
+                      n.cluster=n.cluster)
+      if(is.null(out)) out <- temp
+      else out <- cbind(out, temp)
+    }
+
+    # reorder LOD score columns and make sure that the names are correct
+    colnames(out)[-(1:2)] <- colnames(cross$pheno)[unlist(batches)]
+    out[,-(1:2)] <- out[,colnames(cross$pheno)[pheno.col]]
+    colnames(out)[-(1:2)] <- colnames(cross$pheno)[pheno.col]
+
+    return(out)
+  }
+
+  # multiple phenotype for methods except imp and hk
   if(length(pheno.col)>1 && n.perm <= 0 && 
-     (use=="all.obs" || (method!="imp" && method != "hk" ))) {
+     method!="imp" && method != "hk") {
     # do this by brute force
     out <- scanone(cross, chr, pheno.col[1], model, method,
                    addcovar, intcovar, weights, use, upper, ties.random,
